import h5py
from jax import numpy as jnp
from cyxtrax.common.mesh_object import MeshObject
import json
from pathlib import Path


def from_dict(mesh_dict) -> MeshObject:
    return MeshObject.from_dict(mesh_dict)


def load_atlas(load_path: Path) -> tuple[jnp.ndarray, jnp.ndarray, list[MeshObject]]:
    with h5py.File(load_path, "r") as f:
        maps = jnp.array(f["/maps"][:])
        points = jnp.array(f["/positions"][:])

    mesh_object_dict = load_atlas_dict(load_path)
    mesh_object_list = list(map(from_dict, mesh_object_dict))

    return maps, points, mesh_object_list


def load_atlas_dict(load_path: Path):
    with h5py.File(load_path, "r") as f:
        mesh_object_str = f.attrs["mesh_list"]
        mesh_object_dict = json.loads(mesh_object_str)
        return mesh_object_dict
