from .augemtation_parameters import AugemntationParameter

import xraylib
from jax import numpy as jnp, random


def get_attenuation_coefficient_monochrom(element: str, energy_keV: float) -> float:
    element = xraylib.SymbolToAtomicNumber("C")
    return xraylib.CS_Total(element, energy_keV)


def get_attenuation_coefficient_polychrom(
    element: str, energy_keV: float, bins: int
) -> float:
    energies = jnp.linspace(0, energy_keV, bins, endpoint=True)
    for i in range(1, bins):
        energies = energies.at[i].set(
            get_attenuation_coefficient_monochrom(element, float(energies[i]))
        )
    return energies.sum()


def add_material(
    maps: jnp.ndarray, materilal: str, energy_keV: float, bins: int, key: random.PRNGKey
):
    key4next = key
    function_kwargs = dict()
    input_kwargs = dict(element=materilal, energy_keV=energy_keV, bins=bins)
    attenuation_coefficient = get_attenuation_coefficient_polychrom(**input_kwargs)

    parameters = AugemntationParameter(
        add_material.__globals__["__name__"],
        add_material.__name__,
        function_kwargs,
        input_kwargs,
    )
    maps = maps.at[:].multiply(attenuation_coefficient)

    return maps, parameters, key4next


if __name__ == "__main__":
    mu = get_attenuation_coefficient_polychrom("c", 100, 10)
    print(mu)
