from pathlib import Path
from cyxtrax.simulation import CylindricalProjection, load_mesh
from cyxtrax.common import MeshObject
from cyxtrax.io import record_atlas
from jax import random, numpy as jnp
from jax.scipy.spatial.transform import Rotation


def generate_atlas_from_folder(
    folder: Path,
    key: random.PRNGKey,
    save_folder: Path,
    map_bounding_box: tuple[float] = (-50.0, 50.0),
    number_of_maps: int = 4,
):
    randint_key, postion_key, orientation_key = random.split(key, 3)
    stl_files = list(folder.glob("*.stl"))
    number_of_files = len(stl_files)

    item_id = random.randint(randint_key, (1,), 0, number_of_files, dtype=jnp.int32)
    random_euler = random.uniform(orientation_key, (3,), minval=-180, maxval=180)
    rot: Rotation = Rotation.from_euler("xyz", random_euler)
    stl_path: Path = stl_files[item_id[0]]
    mesh_item = MeshObject(
        stl_path,
        random.truncated_normal(postion_key, -50, 50, (3,))
        + random.uniform(postion_key, (3,), minval=-25, maxval=25),
        rot.as_quat(),
    )
    mesh_list = [mesh_item]
    cycl_proj = CylindricalProjection()
    load_mesh(cycl_proj, mesh_list)
    record_atlas(
        cycl_proj,
        mesh_list,
        stl_path.stem,
        save_folder,
        map_bounding_box,
        number_of_maps,
    )


def merge_atlas_with_data_augmentation(folder: Path, data_augmentation_pipeline):
    pass
