from pathlib import Path

from cyxtrax.io.load_maps import load_atlas
from cyxtrax.io.save_map import add_rays_to_atlas
import numpy as np


def transform_to_rays(file: Path) -> np.ndarray:
    maps, points, _ = load_atlas(file)
    pixel_scale = (maps.shape[0]  - 1.) / maps.shape[0] 

    def cylinder_coordinates(alpha: float, radius: float = 1000.) -> np.ndarray:
        x = np.sin(alpha) * radius
        y = np.cos(alpha) * radius
        point = np.array([x, y, 0.])
        return point
    
    def map_z(point, radius: float = 1000.):
        point = point.reshape((1, 3))
        z = np.linspace(-radius, radius, maps.shape[1]) * np.pi * pixel_scale
        z_array = np.zeros((maps.shape[1], 3))
        z_array[..., 2] = z
        return point + z_array

    
    xyz = list(map(cylinder_coordinates, np.linspace(
        -np.pi * pixel_scale, np.pi * pixel_scale, maps.shape[0], endpoint=False)))
    xyz = np.array(list(map(map_z, xyz)))

    start = points.swapaxes(0, 1)
    
    ray_end = np.expand_dims(xyz, 2) + np.expand_dims(np.expand_dims(start, 0), 0)
    ray_end = np.roll(ray_end, 500, 0)

    ray_start = np.roll(ray_end, 1000, 0)
    ray_start = np.flip(ray_start, 1)
    return ray_start, ray_end


if __name__ == '__main__':
    save_path = Path(r'.\dir2025\00004_smurf.h5')
    maps, points, mesh_object_list = load_atlas(save_path)


    ray_start, ray_end = transform_to_rays(save_path)

    from cyxtrax.io.save_map import add_rays_to_atlas
    from cyxtrax.mapping import map_source_2_cylinder

    add_rays_to_atlas(save_path, ray_start, ray_end)

    index_0, index_1, map_index = 1432, 1111, 11

    source = ray_start[index_0, index_1, map_index]

    values, uvw = map_source_2_cylinder(source, maps, points)
    print(values.shape)
    print(uvw.shape)

    value = values[map_index]
    print(uvw[:2, map_index], values[map_index], maps[index_0, index_1, map_index])

    